package jfftw;

/**     Class containing some useful static method to manipulate
 *      double arrays used in jfftw.
 *      @author Will Hossack, The University of Edinburgh
 * 
 */
public class ArrayUtil extends Object {

    /**   Static method to take two double arrays normally holding
     *    real and imaginary parts and interleave them to a single
     *    array. If imaginary array is null or length zero it is
     *    taken as zero.
     *    @param real the real array
     *    @param imag the imaginary array (can be null, or length zero).
     *    @return <code>double[]</code> interleaved array.
     *    
     */
    public static double[] interleave(double[] real, double[] imag) {
	boolean isImaginary;
	if (imag == null || imag.length == 0) {
	    isImaginary = false;
	}
	else {
	    isImaginary = true;
	}

	double[] data = new double[2*real.length]; // Array of double length
	for(int i = 0; i < real.length; i++) {
	    int j = 2*i;
	    data[j] = real[i];                     // Copy real
	    if (isImaginary) data[j+1] = imag[i];  // Copy imag if exists
	}
	return data;
    }

    /**   Static method to take split array normally holding
     *    real and imaginary parts and interleave them to a single
     *    array. 
     *    @param data the split array
     *    @return <code>double[]</code> interleaved array.
     *    
     */
    public static double[] interleave(double[][] data) {
	return interleave(data[0],data[1]);
    }


    /**   Static method to split an interleaved array into two split
     *    array, returned as a double[2][] array with [0][i] and
     *    [1][i] holding the real / imaginary parts.
     *    @param data the interleaved array.
     *    @return <code>double[][]</code> of size [2][data.length/2]
     */
    public static double[][] split(double[] data) {
	int size = data.length/2;
	double[][] sp = new double[2][size];

	for(int i = 0; i < size; i++) {
	    int j = 2*i;
	    sp[0][i] = data[j];
	    sp[1][i] = data[j + 1];
	}
	return sp;
    }

    /**   Static method to form a split array from a real and
     *    imaginary array. If imaginary array is null or of length
     *    zero, it is assumed to be zero.
     *    <p>
     *    Note the data is not copied to new space.
     *    @param real the real array
     *    @param imag the imaginary array
     *    @return <code>double[][]</code> array in split format.
     */
    public static double[][] split(double[] real, double[] imag) {
	double[][] data = new double[2][];
	data[0] = real;
	if (imag == null || imag.length == 0) {
	    data[1] = new double[real.length];
	}
	else {
	    data[1] = imag;
	}
	return data;
    }

    /**   Static method to get the ith Complex element from an
     *    interleaved array where real part is in 2*i and imaginary
     *    in 2*i + 1.
     *    @param data the interleaved array
     *    @param i the element index
     */
    public static Complex getComplex(double[] data,int i) {
	int j = 2*i;
	return new Complex(data[j],data[j + 1]);
    }

    /**    Static to set the ith component of a interleaved complex
     *     array with a Complex.
     *     @param data the interleaved array
     *     @param i the element index
     *     @param z the value
     */
    public static void setComplex(double[] data, int i, Complex z) {
	int j = 2*i;
	data[j] = z.getReal();
	data[j + 1] = z.getImag();
    }
    
    /**   Static method to get the ith Complex element from a
     *    split array consisting of real and imaginary parts.
     *    @param real the real array.
     *    @param imag the imaginary array.
     *    @param i the element index.
     */
    public static Complex getComplex(double[] real, double[] imag, int i) {
	return new Complex(real[i],imag[i]);
    }


    /**    Static to set the ith componet of split array 
     *     array with a Complex.
     *     @param real the real array
     *     @param imag the imaginary array
     *     @param i the element index
     *     @param z the value
     */
    public static void setComplex(double[] real, double[] imag, int i,
				  Complex z) {
	real[i] = z.getReal();
	imag[i] = z.getImag();
    }



    /**   Static method to get the ith Complex element from a
     *    split array consisting of real and imaginary parts in
     *    a [2][] array
     *    @param split the [2][] array
     *    @param i the element index.
     */
    public static Complex getComplex(double[][] split, int i) {
	return getComplex(split[0],split[1],i);
    }


    /**    Static to set the ith componet of split array 
     *     array with a Complex.
     *     @param split the split array of real and imaginary parts
     *     @param i the element index
     *     @param z the value
     */
    public static void setComplex(double[][] split, int i, Complex z) {
	setComplex(split[0],split[1],i,z);
    }
}
    
    

	
